@@ -1,4 +1,4 @@
-__version__ = "2.0.0"
+__version__ = "2.1.0"
 __author__ = "Hanjin Liu"
 __email__ = "liuhanjin-sc@g.ecc.u-tokyo.ac.jp"
 
@@ -11,8 +11,9 @@ from .core import *
 from .binder import bind
 from .viewer import gui
 from .correlation import *
-from .arrays import ImgArray, LazyImgArray  # for typing
-from . import random
+from .arrays import ImgArray, LazyImgArray, Label  # for typing
+from . import random, io
+from .axes import slicer
 
 r"""
 Inheritance
@@ -31,4 +32,13 @@ logging.getLogger("skimage").setLevel(logging.ERROR)
 logging.getLogger("tifffile").setLevel(logging.ERROR)
 
 
-del logging
\ No newline at end of file
+del logging
+
+# dtypes
+from numpy import (
+	uint8, uint16, uint32, uint64,
+	int8, int16, int32, int64,
+	float16, float32, float64,
+	complex64, complex128,
+	bool_,
+)
