@@ -15,14 +15,14 @@ from .axesmixin import AxesMixin, get_axes_tuple
 from ._utils._skimage import skres
 from ._utils import _misc, _transform, _structures, _filters, _deconv, _corr, _docs
 
-from ..utils.axesop import switch_slice, complement_axes, find_first_appeared, del_axis
-from ..utils.deco import record_lazy, dims_to_spatial_axes, same_dtype
+from ..utils.axesop import slice_axes, switch_slice, complement_axes, find_first_appeared
+from ..utils.deco import check_input_and_output_lazy, dims_to_spatial_axes, same_dtype
 from ..utils.misc import check_nd
-from ..utils.slicer import axis_targeted_slicing, key_repr
-from ..utils.io import get_imsave_meta_from_img, memmap_tif, memmap_mrc
+from ..utils import slicer
+from ..io import imsave
 from ..collections import DataList
 
-from .._types import nDFloat, Coords, Iterable, Dims
+from .._types import nDFloat, Coords, Iterable, Dims, PaddingMode
 from ..axes import ImageAxesError
 from .._const import Const
 from ..array_api import xp
@@ -33,6 +33,7 @@ if TYPE_CHECKING:
 
 class LazyImgArray(AxesMixin):
     additional_props = ["_source", "_metadata", "_name"]
+    
     def __init__(
         self,
         obj: "da.core.Array",
@@ -44,7 +45,7 @@ class LazyImgArray(AxesMixin):
         from dask import array as da
         if not isinstance(obj, da.core.Array):
             raise TypeError(f"The first input must be dask array, got {type(obj)}")
-        self.value = obj
+        self.value: "da.core.Array" = obj
         self.source = source
         self._name = name
         self.axes = axes
@@ -113,32 +114,21 @@ class LazyImgArray(AxesMixin):
             return self.value.chunksize
         
     @property
-    def gb(self):
+    def GB(self) -> float:
+        """Return the array size in GB."""
         return self.value.nbytes / 1e9
     
+    gb = GB  # alias
+    
     def __array__(self):
         # Should not be `self.compute` because in napari Viewer this function is called every time
         # sliders are moved.
         return xp.asnumpy(self.value.compute())
     
     def __getitem__(self, key):
-        if isinstance(key, str):
-            key = axis_targeted_slicing(self.value.ndim, self.axes, key)
-        keystr = key_repr(key) # write down key like "0,*,*"
-        
-        if hasattr(key, "__array__"):
-            # fancy indexing will lose axes information
-            new_axes = None
+        key = slicer.solve_slicer(key, self.axes)
             
-        elif "new" in keystr:
-            # np.newaxis or None will add dimension
-            new_axes = None
-            
-        elif self.axes:
-            del_list = [i for i, s in enumerate(keystr.split(",")) if s not in ("*", "")]
-            new_axes = del_axis(self.axes, del_list)
-        else:
-            new_axes = None
+        new_axes = slice_axes(self.axes, key)
         out = self.__class__(
             self.value[key], 
             name=self.name,
@@ -147,12 +137,14 @@ class LazyImgArray(AxesMixin):
             metadata=self.metadata
         )
         
-        out._getitem_additional_set_info(self, keystr=keystr,
-                                         new_axes=new_axes, key=key)
+        out._getitem_additional_set_info(
+            self, new_axes=new_axes, key=key
+        )
         
         return out
     
     def __neg__(self) -> LazyImgArray:
+        """Invert array."""
         out = self.__class__(-self.value)
         out._set_info(self)
         return out
@@ -201,8 +193,6 @@ class LazyImgArray(AxesMixin):
         elif isinstance(other, self.__class__) and other.dtype.kind != "c":
             other = other.as_float()
             other = other.value
-        elif np.isscalar(other) and other < 0:
-            raise ValueError("Cannot multiply negative value.")
         else:
             other = other
         out = self.value * other
@@ -217,8 +207,6 @@ class LazyImgArray(AxesMixin):
         elif isinstance(other, self.__class__) and other.dtype.kind != "c":
             other = other.as_float()
             other = other.value
-        elif np.isscalar(other) and other < 0:
-            raise ValueError("Cannot multiply negative value.")
         else:
             other = other
         self.value *= other
@@ -263,10 +251,7 @@ class LazyImgArray(AxesMixin):
     
     @property
     def chunk_info(self):
-        if self.axes.is_none():
-            chunk_info = self.chunksize
-        else:
-            chunk_info = ", ".join([f"{s}({o})" for s, o in zip(self.chunksize, self.axes)])
+        chunk_info = ", ".join([f"{s}({o})" for s, o in zip(self.chunksize, self.axes)])
         return chunk_info
     
     def _repr_dict_(self):
@@ -296,18 +281,6 @@ class LazyImgArray(AxesMixin):
             img = arr
         return img
     
-    @property
-    def data(self):
-        warn("'data' should no longer be used and will be removed soon. Use 'img.compute()' instead.", 
-             DeprecationWarning)
-        return self.compute()
-    
-    @property
-    def img(self):
-        warn("'img' is renamed to 'value' for compatibility with ImgArray and will be removed soon.", 
-             DeprecationWarning)
-        return self.value
-    
     def release(self, update: bool = True) -> LazyImgArray:
         """
         Compute all the tasks and store the data in memory map, and read it as a dask array
@@ -336,27 +309,19 @@ class LazyImgArray(AxesMixin):
         _, ext = os.path.splitext(save_path)
         
         if ext == "":
-            save_path += ".tif"
-            ext = ".tif"
+            if self.source is not None:
+                ext = self.source.suffix
+            else:
+                ext = ".tif"
+            save_path += ext
         
-        if ext in (".tif", ".tiff"):
-            mmap_func = memmap_tif
-        elif ext in (".mrc", ".map"):
-            mmap_func = memmap_mrc
-        else:
-            raise ValueError(f"Unsupported file type '{ext}'.")
-    
         if os.sep not in save_path:
             save_path = os.path.join(self.source.parent, save_path)
         if dtype is None:
             dtype = self.dtype
         
         self = self.as_img_type(dtype).sort_axes()
-        imsave_kwargs = get_imsave_meta_from_img(self, update_lut=False)
-        
-        mmap_func(
-            self.value, save_path, shape=self.shape, dtype=self.dtype, **imsave_kwargs
-        )
+        imsave(save_path, self, lazy=True)
         
         return None
     
@@ -413,7 +378,7 @@ class LazyImgArray(AxesMixin):
         
         out = getattr(self.value, funcname)(*args, **kwargs)
         out = self.__class__(out)
-        new_axes = "inherit" if out.shape == self.shape else None
+        new_axes = self._INHERIT if out.shape == self.shape else None
         out._set_info(self, new_axes=new_axes)
         return out
     
@@ -606,7 +571,7 @@ class LazyImgArray(AxesMixin):
     
     @_docs.copy_docs(ImgArray.erosion)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def erosion(
         self,
         radius: float = 1,
@@ -623,11 +588,11 @@ class LazyImgArray(AxesMixin):
             c_axes=c_axes,
             depth=_ceilint(radius),
             kwargs=dict(footprint=disk)
-            )
+        )
         
     @_docs.copy_docs(ImgArray.dilation)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def dilation(
         self,
         radius: float = 1,
@@ -644,11 +609,11 @@ class LazyImgArray(AxesMixin):
             c_axes=c_axes,
             depth=_ceilint(radius),
             kwargs=dict(footprint=disk)
-            )
+        )
     
     @_docs.copy_docs(ImgArray.opening)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def opening(
         self,
         radius: float = 1,
@@ -665,11 +630,11 @@ class LazyImgArray(AxesMixin):
             c_axes=c_axes,
             depth=_ceilint(radius)*2,
             kwargs=dict(footprint=disk)
-            )
+        )
     
     @_docs.copy_docs(ImgArray.closing)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def closing(
         self,
         radius: float = 1,
@@ -686,12 +651,12 @@ class LazyImgArray(AxesMixin):
             c_axes=c_axes,
             depth=_ceilint(radius)*2,
             kwargs=dict(footprint=disk)
-            )
+        )
     
     @_docs.copy_docs(ImgArray.gaussian_filter)
     @dims_to_spatial_axes
     @same_dtype(asfloat=True)
-    @record_lazy
+    @check_input_and_output_lazy
     def gaussian_filter(
         self,
         sigma: nDFloat = 1.0,
@@ -706,12 +671,34 @@ class LazyImgArray(AxesMixin):
             c_axes=c_axes,
             depth=depth,
             kwargs=dict(sigma=sigma),
-            )
+        )
+    
+    @_docs.copy_docs(ImgArray.spline_filter)
+    @dims_to_spatial_axes
+    @same_dtype(asfloat=True)
+    @check_input_and_output_lazy
+    def spline_filter(
+        self,
+        order: int = 3,
+        mode: PaddingMode = "mirror", 
+        *,
+        dims: Dims = None,
+        update: bool = False,
+    ):
+        # TODO: need test
+        depth = order
+        return self._apply_map_overlap(
+            _filters.spline_filter,
+            c_axes=complement_axes(dims, self.axes), 
+            depth=depth,
+            args=(order, np.float32, mode),
+        )
+    
     
     @_docs.copy_docs(ImgArray.median_filter)
     @dims_to_spatial_axes
     @same_dtype
-    @record_lazy
+    @check_input_and_output_lazy
     def median_filter(
         self,
         radius: float = 1,
@@ -730,7 +717,7 @@ class LazyImgArray(AxesMixin):
     @_docs.copy_docs(ImgArray.mean_filter)
     @same_dtype(asfloat=True)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def mean_filter(
         self,
         radius: float = 1,
@@ -750,7 +737,7 @@ class LazyImgArray(AxesMixin):
     @_docs.copy_docs(ImgArray.convolve)
     @dims_to_spatial_axes
     @same_dtype(asfloat=True)
-    @record_lazy
+    @check_input_and_output_lazy
     def convolve(
         self,
         kernel,
@@ -775,7 +762,7 @@ class LazyImgArray(AxesMixin):
     @_docs.copy_docs(ImgArray.edge_filter)
     @dims_to_spatial_axes
     @same_dtype
-    @record_lazy
+    @check_input_and_output_lazy
     def edge_filter(
         self,
         method: str = "sobel", 
@@ -804,7 +791,7 @@ class LazyImgArray(AxesMixin):
     @_docs.copy_docs(ImgArray.laplacian_filter)
     @dims_to_spatial_axes
     @same_dtype
-    @record_lazy
+    @check_input_and_output_lazy
     def laplacian_filter(
         self,
         radius: int = 1,
@@ -824,7 +811,7 @@ class LazyImgArray(AxesMixin):
     @_docs.copy_docs(ImgArray.affine)
     @dims_to_spatial_axes
     @same_dtype(asfloat=True)
-    @record_lazy
+    @check_input_and_output_lazy
     def affine(
         self, 
         matrix=None,
@@ -856,7 +843,7 @@ class LazyImgArray(AxesMixin):
     @_docs.copy_docs(ImgArray.kalman_filter)
     @dims_to_spatial_axes
     @same_dtype(asfloat=True)
-    @record_lazy
+    @check_input_and_output_lazy
     def kalman_filter(self, gain: float = 0.8, noise_var: float = 0.05, *, along: str = "t", 
                       dims: Dims = None, update: bool = False) -> LazyImgArray:
         if self.axisof(along) != 0:
@@ -864,13 +851,13 @@ class LazyImgArray(AxesMixin):
         
         return self._apply_map_blocks(
             _filters.kalman_filter, 
-            c_axes=complement_axes(along + dims, self.axes), 
+            c_axes=complement_axes([along] + dims, self.axes), 
             args=(gain, noise_var)
-            )
+        )
     
     @_docs.copy_docs(ImgArray.fft)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def fft(self, *, shape: int | Iterable[int] | str = "same", shift: bool = True, 
             dims: Dims = None) -> LazyImgArray:
         from dask import array as da
@@ -889,7 +876,7 @@ class LazyImgArray(AxesMixin):
 
     @_docs.copy_docs(ImgArray.ifft)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def ifft(self, real:bool=True, *, shift:bool=True, dims=None) -> LazyImgArray:
         from dask import array as da
         axes = [self.axisof(a) for a in dims]
@@ -906,7 +893,7 @@ class LazyImgArray(AxesMixin):
     
     @_docs.copy_docs(ImgArray.power_spectra)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def power_spectra(self, shape = "same", norm: bool = False, zero_norm: bool = False, *,
                       dims: Dims = None) -> LazyImgArray:
         freq = self.fft(dims=dims, shape=shape)
@@ -925,10 +912,7 @@ class LazyImgArray(AxesMixin):
         return tuple(self.chunksizeof(a) for a in axes)
         
     def transpose(self, axes):
-        if self.axes.is_none():
-            new_axes = None
-        else:
-            new_axes = "".join([self.axes[i] for i in list(axes)])
+        new_axes = [self.axes[i] for i in list(axes)]
         out = self.__class__(self.value.transpose(axes))
         out._set_info(self, new_axes=new_axes)
         return out
@@ -939,7 +923,7 @@ class LazyImgArray(AxesMixin):
     
     @_docs.copy_docs(LabeledArray.crop_center)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def crop_center(self, scale=0.5, *, dims=2) -> LazyImgArray:
         # check scale
         if hasattr(scale, "__iter__") and len(scale) == 3 and len(dims) == 2:
@@ -962,7 +946,7 @@ class LazyImgArray(AxesMixin):
     
     @_docs.copy_docs(ImgArray.tiled_lowpass_filter)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def tiled_lowpass_filter(self, cutoff: float = 0.2, order: int = 2, overlap: int = 16, *,
                              dims: Dims = None, update: bool = False) -> LazyImgArray:
         from ._utils._skimage import _get_ND_butterworth_filter
@@ -1001,7 +985,7 @@ class LazyImgArray(AxesMixin):
             projection = getattr(da, method)(self.value, axis=tuple(axisint))
         
         out = self.__class__(projection)
-        out._set_info(self, del_axis(self.axes, axisint))
+        out._set_info(self, self.axes.drop(axisint))
         return out
     
     @_docs.copy_docs(ImgArray.binning)
@@ -1072,7 +1056,7 @@ class LazyImgArray(AxesMixin):
     
     @_docs.copy_docs(ImgArray.drift_correction)
     @same_dtype(asfloat=True)
-    @record_lazy
+    @check_input_and_output_lazy
     @dims_to_spatial_axes
     def drift_correction(self, shift: Coords = None, ref: ImgArray = None, *, 
                          zero_ave: bool = True, along: str = None, dims: Dims = 2, 
@@ -1134,7 +1118,7 @@ class LazyImgArray(AxesMixin):
     
     @_docs.copy_docs(ImgArray.pad)
     @dims_to_spatial_axes
-    @record_lazy
+    @check_input_and_output_lazy
     def pad(self, pad_width, mode: str = "constant", *, dims: Dims = None, **kwargs) -> LazyImgArray:
         pad_width = _misc.make_pad(pad_width, dims, self.axes, **kwargs)
         padimg = np.pad(self.value, pad_width, mode, **kwargs)
@@ -1213,10 +1197,10 @@ class LazyImgArray(AxesMixin):
         return out
     
     def _process_output(self, input: LazyImgArray, args: tuple, kwargs: dict):
-        if "axis" in kwargs.keys() and not input.axes.is_none():
-            new_axes = del_axis(input.axes, kwargs["axis"])
+        if "axis" in kwargs.keys():
+            new_axes = input.axes.drop(kwargs["axis"])
         else:
-            new_axes = "inherit"
+            new_axes = self._INHERIT
         self._set_info(input, new_axes=new_axes)
         return None
         
@@ -1312,13 +1296,12 @@ class LazyImgArray(AxesMixin):
         self._set_info(other, kwargs["new_axes"])
         return None
     
-    def _set_info(self, other: LazyImgArray, new_axes: str = "inherit"):
+    def _set_info(self, other: LazyImgArray, new_axes: Any = AxesMixin._INHERIT):
         self._set_additional_props(other)
         # set axes
         try:
-            if new_axes != "inherit":
+            if new_axes is not self._INHERIT:
                 self.axes = new_axes
-                self.set_scale(other)
             else:
                 self.axes = other.axes.copy()
         except ImageAxesError:
