@@ -12,12 +12,12 @@ import numpy as np
 from numpy.typing import ArrayLike, DTypeLike, _ShapeLike
 from functools import wraps
 
-from .utils.io import open_img, open_as_dask, get_scale_from_meta, open_mrc, open_tif
+from . import io
 from .utils import gauss
 from .utils.slicer import *
 from ._types import *
 
-from .axes import ImageAxesError
+from .axes import ImageAxesError, broadcast, Axes
 from .collections import DataList
 from .arrays.bases import MetaArray
 from .arrays import ImgArray, LazyImgArray
@@ -31,13 +31,15 @@ __all__ = [
     "empty", 
     "ones", 
     "full",
+    "arange",
     "gaussian_kernel", 
     "circular_mask", 
     "imread", 
     "imread_collection", 
     "lazy_imread", 
     "read_meta", 
-    "sample_image"
+    "sample_image",
+    "broadcast_arrays"
 ]
 
 # TODO: 
@@ -55,6 +57,7 @@ shared_docs = \
     """    
     
 def write_docs(func):
+    """Add doc for numpy function."""
     func.__doc__ = re.sub(r"{}", shared_docs, func.__doc__)
     return func
 
@@ -178,13 +181,14 @@ def aslazy(
 
 _P = ParamSpec("_P")
 
-def inject_numpy_function(func: Callable[_P, np.ndarray]) -> Callable[_P, ImgArray]:
+def _inject_numpy_function(func: Callable[_P, Any | None]) -> Callable[_P, ImgArray]:
     npfunc: Callable = getattr(np, func.__name__)
     @wraps(func)
     def _func(*args, **kwargs):
         axes = kwargs.pop("axes", None)
         name = kwargs.pop("name", None)
         return asarray(npfunc(*args, **kwargs), name=name, axes=axes)
+    
     _func.__doc__ = (
         f"""
         impy version of numpy.{func.__name__}. This function has additional parameters ``axes``
@@ -203,22 +207,27 @@ def inject_numpy_function(func: Callable[_P, np.ndarray]) -> Callable[_P, ImgArr
     _func.__annotations__.update({"return": ImgArray})
     return _func
 
-@inject_numpy_function
+@_inject_numpy_function
 def zeros(shape: _ShapeLike, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
     
-@inject_numpy_function
+@_inject_numpy_function
 def empty(shape: _ShapeLike, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
 
-@inject_numpy_function
+@_inject_numpy_function
 def ones(shape: _ShapeLike, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
 
-@inject_numpy_function
+@_inject_numpy_function
 def full(shape: _ShapeLike, fill_value: Any, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
 
+@_inject_numpy_function
+def arange(stop: int, dtype: DTypeLike = ..., like: Any = ...): ...
 
-def gaussian_kernel(shape: _ShapeLike, 
-                    sigma: nDFloat = 1.0,
-                    peak: float = 1.0) -> ImgArray:
+
+def gaussian_kernel(
+    shape: _ShapeLike, 
+    sigma: nDFloat = 1.0,
+    peak: float = 1.0,
+) -> ImgArray:
     """
     Make an Gaussian kernel or Gaussian image.
 
@@ -307,6 +316,19 @@ def sample_image(name: str) -> ImgArray:
         out = out.sort_axes()
     return out
 
+def broadcast_arrays(*arrays: MetaArray) -> list[MetaArray]:
+    axes_list: list[Axes] = []
+    shapes: dict[str, int] = {}
+    for arr in arrays:
+        axes_list.append(arr.axes)
+        for a, s in zip(arr.axes, arr.shape):
+            shapes[a] = s
+    axes = broadcast(*axes_list)
+    shape = tuple(shapes[a] for a in axes)
+    
+    out = [a.broadcast_to(shape, axes) for a in arrays]
+    return out
+
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
 #   Imread functions
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
@@ -314,14 +336,14 @@ def sample_image(name: str) -> ImgArray:
 def imread(
     path: str,
     dtype: DTypeLike = None,
-    key: str = None,
+    key: AxesTargetedSlicer | None = None,
     *, 
     name: str | None = None,
     squeeze: bool = False,
 ) -> ImgArray:
     r"""
-    Load image(s) from a path. You can read list of images from directories with wildcards or ``"$"``
-    in ``path``.
+    Load image(s) from a path. You can read list of images from directories with 
+    wildcards or ``"$"`` in ``path``.
 
     Parameters
     ----------
@@ -329,10 +351,10 @@ def imread(
         Path to the image or directory.
     dtype : Any type that np.dtype accepts
         Data type of images.
-    key : str, optional
-        If not None, image is read in a memory-mapped array first, and only ``img[key]`` is returned.
-        Only axis-targeted slicing is supported. This argument is important when reading a large
-        file.
+    key : AxesTargetedSlicer, optional
+        If not None, image is read in a memory-mapped array first, and only 
+        ``img[key]`` is returned. Only axis-targeted slicing is supported. This
+        argument is important when reading a large file.
     name : str, optional
         Name of array.
     squeeze : bool, default is False
@@ -355,8 +377,6 @@ def imread(
         
     """    
     path = str(path)
-    if not os.path.exists(path):
-        raise ValueError(f"Path does not exist: {path}.")
     is_memmap = (key is not None)
     
     if "$" in path:
@@ -368,28 +388,35 @@ def imread(
     
     # read tif metadata
     if not is_memmap:
-        size = os.path.getsize(path)/1e9
+        size = os.path.getsize(path) / 1e9
         if size > Const["MAX_GB"]:
             raise MemoryError(f"Too large {size:.2f} GB")
 
-    meta, img = open_img(path, memmap=is_memmap)
+    image_data = io.imread(path, memmap=is_memmap)
 
-    axes = meta["axes"]
-    metadata = meta["ijmeta"]
+    img = image_data.image
+    axes = image_data.axes
+    scale = image_data.scale
+    scale_unit = image_data.unit
+    metadata = image_data.metadata
+    labels = image_data.labels
+    
+    if not isinstance(scale_unit, dict):
+        scale_unit = {a: scale_unit for a in "zyx"}
         
-    if is_memmap:
-        sl = axis_targeted_slicing(img.ndim, axes, key)
+    if is_memmap and axes is not None:
+        sl = solve_slicer(key, Axes(axes))
         axes = "".join(a for a, k in zip(axes, sl) if not isinstance(k, int))
         img = np.asarray(img[sl], dtype=dtype)
     
     self = ImgArray(img, name=name, axes=axes, source=path, metadata=metadata)
         
-    # In case the image is in yxc-order. This sometimes happens.
-    if "c" in self.axes and self.shape.c > self.shape.x:
-        self: ImgArray = np.moveaxis(self, -1, -3)
-        _axes = self.axes.axes
-        _axes = _axes[:-3] + "cyx"
-        self.axes = _axes
+    if "c" in self.axes:
+        if self.shape.c > self.shape.x:
+            # In case the image is in yxc-order. This sometimes happens.
+            self: ImgArray = np.moveaxis(self, -1, -3)
+        if labels is not None:
+            self.set_axis_label(c=labels)
     
     if dtype is None:
         dtype = self.dtype
@@ -397,19 +424,15 @@ def imread(
     if squeeze:
         self = np.squeeze(self)
         
-    if self.axes.is_none():
-        return self
-    else:
-        # read lateral scale if possible
-        scale = get_scale_from_meta(meta)
-        
-        # if key="y=0", ImageAxisError happens here because image loses y-axis. We have to set scale
-        # one by one.
-        for k, v in scale.items():
-            if k in self.axes:
-                self.set_scale({k: v})
-        
-        return self.sort_axes().as_img_type(dtype) # arrange in tzcyx-order
+    # if key="y=0", ImageAxisError happens here because image loses y-axis. We have to set scale
+    # one by one.
+    for k, v in scale.items():
+        if k in self.axes:
+            self.set_scale({k: v})
+            if k in "zyx":
+                self.axes[k].unit = scale_unit[k]
+    
+    return self.sort_axes().as_img_type(dtype) # arrange in tzcyx-order
 
 def _imread_glob(path: str, squeeze: bool = False, **kwargs) -> ImgArray:
     """
@@ -434,7 +457,7 @@ def _imread_glob(path: str, squeeze: bool = False, **kwargs) -> ImgArray:
     if len(imgs) == 0:
         raise RuntimeError("Could not read any images.")
     
-    out = np.stack(imgs, axis="p")
+    out: ImgArray = np.stack(imgs, axis="p")
     if squeeze:
         out = np.squeeze(out)
     try:
@@ -442,13 +465,13 @@ def _imread_glob(path: str, squeeze: bool = False, **kwargs) -> ImgArray:
         out.source = base
     except Exception:
         pass
-    out.temp = paths
+
     return out
 
 def _imread_stack(
     path: str, 
     dtype: DTypeLike = None,
-    key: str = None,
+    key: AxesTargetedSlicer | None = None,
     squeeze: bool = False
 ):
     r"""
@@ -643,24 +666,35 @@ def lazy_imread(
     if "*" in path:
         return _lazy_imread_glob(path, chunks=chunks, squeeze=squeeze)
     
-    # read tif metadata
-    meta, img = open_as_dask(path, chunks)
-    axes = meta["axes"]
-    metadata = meta["ijmeta"]
-        
+    # read as a dask array
+    image_data = io.imread_dask(path, chunks)
+    img = image_data.image
+    axes = image_data.axes
+    scale = image_data.scale
+    spatial_scale_unit = image_data.unit
+    metadata = image_data.metadata
+    labels = image_data.labels
+    
     if squeeze:
         axes = "".join(a for i, a in enumerate(axes) if img.shape[i] > 1)
         img = np.squeeze(img)
     
     self = LazyImgArray(img, name=name, axes=axes, source=path, metadata=metadata)
+
+    # read scale if possible
+    self.set_scale(**scale)
     
-    if self.axes.is_none():
-        return self
-    else:
-        # read lateral scale if possible
-        scale = get_scale_from_meta(meta)
-        self.set_scale(**scale)
-        return self.sort_axes()
+    
+    if "c" in self.axes and labels is not None:
+        self.set_axis_label(c=labels)
+    
+    for k, v in scale.items():
+        if k in self.axes:
+            self.set_scale({k: v})
+            if k in "zyx":
+                self.axes[k].unit = spatial_scale_unit.get(k)
+    
+    return self.sort_axes()
 
 
 def _lazy_imread_glob(path: str, squeeze: bool = False, **kwargs) -> LazyImgArray:
@@ -714,20 +748,12 @@ def read_meta(path: str) -> dict[str]:
     Returns
     -------
     dict
-        Dictionary of metadata with following keys.
-        - "axes": axes information
-        - "ijmeta": ImageJ metadata
-        - "tags": tiff tags
-        
+        Dictionary of keys {"axes", "scale", "metadata"}        
     """    
     path = str(path)
-    _, fext = os.path.splitext(path)
-    meta, _ = open_as_dask(path, chunks="default")
-    if fext in (".tif", ".tiff"):
-        meta = open_tif(path)
-    elif fext in (".mrc", ".rec", ".map"):
-        meta = open_mrc(path)
-    else:
-        raise ValueError("Unsupported file extension.")
-    
-    return meta
+    image_data = io.imread_dask(path, chunks="default")
+    return {
+        "axes": image_data.axes,
+        "scale": image_data.scale,
+        "metadata": image_data.metadata
+    }
