@@ -1,33 +1,71 @@
+from __future__ import annotations
+from typing import TYPE_CHECKING, Callable, Literal, TypeVar, overload
 from functools import wraps
 import numpy as np
 import inspect
-import re
 from ..array_api import xp
 
+if TYPE_CHECKING:
+    from ..arrays import LazyImgArray, ImgArray
+    from ..arrays.axesmixin import AxesMixin
+    from typing_extensions import ParamSpec
+    _P = ParamSpec("_P")
+    _R = TypeVar("_R")
+
 __all__ = [
-    "record",
-    "record_lazy",
+    "check_input_and_output",
+    "check_input_and_output_lazy",
     "same_dtype",
     "dims_to_spatial_axes",
 ]
-            
-    
-def record(func=None, *, inherit_label_info=False, only_binary=False, need_labels=False):
-    def f(func):
+
+
+@overload
+def check_input_and_output(
+    func: Literal[None],
+    *, 
+    inherit_label_info: bool = False,
+    only_binary: bool = False,
+    need_labels: bool = False,
+) -> Callable[[Callable[_P, _R]], Callable[_P, _R]]:
+    ...
+
+@overload
+def check_input_and_output(
+    func: Callable[_P, _R],
+    *, 
+    inherit_label_info: bool = False,
+    only_binary: bool = False,
+    need_labels: bool = False,
+) -> Callable[_P, _R]:
+    ...
+
+def check_input_and_output(
+    func=None,
+    *, 
+    inherit_label_info=False,
+    only_binary=False,
+    need_labels=False
+):
+    def f(func: Callable[_P, _R]) -> Callable[_P, _R]:
         @wraps(func)
-        def _record(self, *args, **kwargs):
+        def _func(self: ImgArray, *args, **kwargs):
             # check requirements of the ongoing function.
             if only_binary and self.dtype != bool:
-                raise TypeError(f"Cannot run {func.__name__} with non-binary image.")
-            if need_labels and not hasattr(self, "labels"):
-                raise AttributeError(f"Function {func.__name__} needs labels."
-                                    " Add labels to the image first.")
+                raise TypeError(
+                    f"Cannot run {func.__name__!r} with non-binary image."
+                )
+            if need_labels and not self.labels is not None:
+                raise ValueError(
+                    f"Function {func.__name__!r} needs labels. Add labels to the "
+                    "image first."
+                )
             
             out = func(self, *args, **kwargs)
-            
+
             if type(out) in (np.ndarray, xp.ndarray):
                 out = xp.asnumpy(out).view(self.__class__)
-            
+
             ifupdate = kwargs.pop("update", False)
             if inherit_label_info:
                 out.labels._set_info(self.labels)
@@ -37,15 +75,23 @@ def record(func=None, *, inherit_label_info=False, only_binary=False, need_label
                 pass
                     
             ifupdate and self._update(out)
-            
+
             return out
-        return _record
+        return _func
     return f if func is None else f(func)
 
-def record_lazy(func=None, *, only_binary=False):
-    def f(func):
+@overload
+def check_input_and_output_lazy(func: Callable[_P, _R], *, only_binary: bool = False) -> Callable[_P, _R]:
+    ...
+
+@overload
+def check_input_and_output_lazy(func: Literal[None], *, only_binary: bool = False) -> Callable[[Callable[_P, _R], Callable[_P, _R]]]:
+    ...
+    
+def check_input_and_output_lazy(func=None, *, only_binary=False):
+    def f(func: Callable[_P, _R]) -> Callable[_P, _R]:
         @wraps(func)
-        def _record(self, *args, **kwargs):
+        def _record(self: LazyImgArray, *args, **kwargs):
             if only_binary and self.dtype != bool:
                 raise TypeError(f"Cannot run {func.__name__} with non-binary image.")
             
@@ -68,7 +114,15 @@ def record_lazy(func=None, *, only_binary=False):
         return _record
     return f if func is None else f(func)
 
-def same_dtype(func=None, asfloat=False):
+@overload
+def same_dtype(func: Callable[_P, _R], asfloat: bool = False) -> Callable[_P, _R]:
+    ...
+
+@overload
+def same_dtype(func: Literal[None], asfloat: bool = False) -> Callable[[Callable[_P, _R]], Callable[_P, _R]]:
+    ...
+    
+def same_dtype(func=None, asfloat: bool = False):
     """
     Decorator to assure output image has the same dtype as the input image. 
     This decorator is compatible with both ImgArray and LazyImgArray.
@@ -78,20 +132,20 @@ def same_dtype(func=None, asfloat=False):
     asfloat : bool, optional
         If input image should be converted to float first, by default False
     """    
-    def f(func):
+    def f(func: Callable[_P, _R]) -> Callable[_P, _R]:
         @wraps(func)
-        def _same_dtype(self, *args, **kwargs):
+        def _same_dtype(self: ImgArray, *args, **kwargs):
             dtype = self.dtype
             if asfloat and self.dtype.kind in "ui":
                 self = self.as_float()
-            out = func(self, *args, **kwargs)
+            out: ImgArray = func(self, *args, **kwargs)
             out = out.as_img_type(dtype)
             return out
         return _same_dtype
     return f if func is None else f(func)
 
 
-def dims_to_spatial_axes(func):
+def dims_to_spatial_axes(func: Callable[_P, _R]) -> Callable[_P, _R]:
     """
     Decorator to convert input `dims` to correct spatial axes. Compatible with ImgArray and
     LazyImgArray
@@ -102,11 +156,13 @@ def dims_to_spatial_axes(func):
     dims="ty" -> "ty"
     """    
     @wraps(func)
-    def _dims_to_spatial_axes(self, *args, **kwargs):
-        dims = kwargs.get("dims", 
-                          inspect.signature(func).parameters["dims"].default)
+    def _dims_to_spatial_axes(self: AxesMixin, *args, **kwargs):
+        dims = kwargs.get(
+            "dims", 
+            inspect.signature(func).parameters["dims"].default
+        )
         if dims is None or dims == "":
-            dims = len([a for a in "zyx" if a in self._axes])
+            dims = len([a for a in "zyx" if a in self.axes])
             if dims not in (2, 3):
                 raise ValueError(
                     f"Image spatial dimension must be 2 or 3, but {dims} was detected. If "
@@ -115,25 +171,11 @@ def dims_to_spatial_axes(func):
                     )
             
         if isinstance(dims, int):
-            s_axes = "".join([a for a in "zyx" if a in self._axes])[-dims:]
+            s_axes = [a for a in "zyx" if a in self.axes][-dims:]
         else:
-            s_axes = str(dims)
+            s_axes = list(dims)
         
         kwargs["dims"] = s_axes # update input
         return func(self, *args, **kwargs)
     
     return _dims_to_spatial_axes
-
-def _safe_str(obj):
-    try:
-        if isinstance(obj, float):
-            s = f"{obj:.3g}"
-        else:
-            s = str(obj)
-        s = re.sub("\n", ";", s)
-        if len(s) > 20:
-            return str(type(obj))
-        else:
-            return s
-    except Exception:
-        return str(type(obj))
