@@ -1,10 +1,11 @@
 from __future__ import annotations
-import numpy as np
 import re
 from functools import lru_cache
+from typing import Any, Mapping
 from .._types import Slices
+from ..axes import Slicer, Axes
 
-__all__ = ["str_to_slice", "key_repr", "axis_targeted_slicing"]
+__all__ = ["str_to_slice", "axis_targeted_slicing", "solve_slicer"]
 
 def _range_to_list(v: str) -> list[int]:
     """
@@ -33,30 +34,9 @@ def str_to_slice(v: str) -> list[int] | slice | int:
         sl = int(v)
     return sl
 
-def key_repr(key):
-    keylist = []
-        
-    if isinstance(key, tuple):
-        _keys = key
-    elif hasattr(key, "__array__"):
-        _keys = ("array",)
-    else:
-        _keys = (key,)
-    
-    for s in _keys:
-        if isinstance(s, (slice, list, np.ndarray)):
-            keylist.append("*")
-        elif s is None:
-            keylist.append("new")
-        elif s is ...:
-            keylist.append("...")
-        else:
-            keylist.append(str(s))
-    
-    return ",".join(keylist)
 
 @lru_cache
-def axis_targeted_slicing(ndim: int, axes: str, string: str) -> Slices:
+def axis_targeted_slicing(axes: tuple[str, ...], string: str) -> Slices:
     """
     Make a conventional slices from an axis-targeted slicing string.
 
@@ -67,26 +47,43 @@ def axis_targeted_slicing(ndim: int, axes: str, string: str) -> Slices:
     axes : str
         Axes of input ndarray.
     string : str
-        Axis-targeted slicing string. If an axis that does not exist in `axes` is contained,
-        this function will raise ValueError.
+        Axis-targeted slicing string. If an axis that does not exist in `axes` is
+        contained, this function will raise ValueError.
 
     Returns
     -------
     slices
     """    
     keylist = re.sub(" ", "", string).split(";")
-    sl_list = [slice(None)]*ndim
+    dict_slicer: dict[str, Any] = {}
     
     for k in keylist:
         if k.count("=") != 1:
-            raise ValueError(f"Informal axis-targeted slicing: {k}")
+            raise ValueError(f"Informal axis-targeted slicing: {k!r}.")
         axis, sl_str = k.split("=")
-        i = axes.find(axis)
-        if i < 0:
-            raise ValueError(f"Axis '{axis}' does not exist ({axes}).")
         try:
-            sl_list[i] = str_to_slice(sl_str)
+            sl = str_to_slice(sl_str)
         except ValueError:
-            raise ValueError(f"Informal axis-targeted slicing: {string}")
+            raise ValueError(f"Informal axis-targeted slicing: {k!r}.")
+        else:
+            dict_slicer[axis] = sl
     
-    return tuple(sl_list)
\ No newline at end of file
+    return dict_to_slice(dict_slicer, axes)
+
+def dict_to_slice(sl: dict[str, Any], axes: tuple[str, ...]):
+    sl_list = [slice(None)] * len(axes)
+    
+    for k, v in sl.items():
+        idx = axes.index(k)
+        sl_list[idx] = v
+    
+    return tuple(sl_list)
+
+def solve_slicer(key: Any, axes: Axes):
+    if isinstance(key, str):
+        key = axis_targeted_slicing(tuple(axes), key)
+    
+    elif isinstance(key, (Mapping, Slicer)):
+        key = axes.create_slice(key)
+    
+    return key
\ No newline at end of file
